/*
 * 	UCBxI2C.c
 *
 *  Created on: 16.11.2014
 *  Author: Gerrit Buhe, DL9GFA
 */

#include <msp430.h>
#include <stdint.h>
#include "UCBxI2C.h"

uint8_t RxBuffer[4];
uint8_t TxBuffer[4];
volatile uint8_t RPT_Flag=0; 				//enables repeated start when 1
uint8_t *pTxData;                     		//pointer to TX data
uint8_t *pRxData;                     		//pointer to RX data
int8_t RxByteCtr,TxByteCtr;


void UCBxI2C_init( void)
{
	P1SEL |= BIT6 + BIT7;                   //assign I2C pins to USCI_B0
	P1SEL2|= BIT6 + BIT7;                   //assign I2C pins to USCI_B0

	UCB0CTL1 = UCSSEL_2 + UCSWRST;          //use SMCLK, hold interface in SW reset mode
	UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;   //I2C, Master, synchronous mode
	UCB0BR0 = 10;                           //f_SCL = SMCLK/10 = 800kHz
	UCB0BR1 = 0;
	UCB0CTL1 &= ~UCSWRST;                   //start operation by disarm SW reset
	IE2 |= UCB0TXIE + UCB0RXIE;				//arm interrupts
}


void UCBxI2C_read( uint8_t device_address, uint8_t reg_address, uint8_t byte_count, uint8_t *rxbuf)
{
	while( UCB0STAT & UCBBUSY);				//wait if any I2C communication is still in progress
	UCB0I2CSA = device_address;				//set slave address to read from
	TxByteCtr = 1;							//send register address to read from
	pTxData   = TxBuffer;    				//start of auxilary TX buffer
	*pTxData  = reg_address;	    		//put register address to auxilary TX buffer
	RPT_Flag  = 1;							//send without stop condition
	UCB0CTL1 |= UCTR + UCTXSTT;             //I2C TX, start condition
	while( UCB0CTL1 & UCTXSTT);				//is slave address acknowledged?

	RxByteCtr = byte_count-1;
	pRxData   = rxbuf;    					//start of RX buffer
	UCB0CTL1 &= ~UCTR;						//RX mode now
	UCB0CTL1 |= UCTXSTT;                    //I2C start condition
}


void UCBxI2C_write(uint8_t device_address, uint8_t reg_address, uint8_t byte_count, uint8_t *txbuf)
{
	while( UCB0STAT & UCBBUSY);				//wait if any I2C communication is still in progress
	UCB0I2CSA = device_address;				//set address to send to
	TxByteCtr = byte_count+2;				//number of bytes to write, one more for reg_address
	pTxData   = TxBuffer;    				//start of auxilary TX buffer
	*pTxData  = reg_address;	    		//put register address into auxilary TX buffer
	UCB0CTL1 |= UCTR + UCTXSTT;             //I2C TX, start condition
	pTxData   = txbuf;    					//set ptr this time to payload buffer
}


// USCI_B0 I2C ISR
#pragma vector = USCIAB0TX_VECTOR
__interrupt void USCIAB0TX_ISR(void)
{
	if( IFG2 & UCB0TXIFG)					//master transmit?
	{
		TxByteCtr--;                      	//decrement TX byte counter
		if (TxByteCtr)                      //check TX byte counter
			UCB0TXBUF = *pTxData++;         //load TX buffer
		else
		{
			UCB0TXBUF = *pTxData++;         //load last data to send
			if(RPT_Flag == 0)
			{
				UCB0CTL1 |= UCTXSTP;        //no Repeated Start --> stop condition
			}
			else
				RPT_Flag = 0;				//no stop condition for repeated start, reset flag

			//_bic_SR_register_on_exit(CPUOFF); //exit LPM0
		}
	}
	else if ( IFG2 & UCB0RXIFG)	//master receive!
	{
		RxByteCtr--;                        //decrement RX byte counter
		if (RxByteCtr)
			*pRxData++ = UCB0RXBUF;         //move RX data to address pRxData
		else
		{
			*pRxData++ = UCB0RXBUF;         //move final RX data to pRxData
			if(RPT_Flag == 0)
			{
				UCB0CTL1 |= UCTXSTP;        //no Repeated Start --> stop condition
			}
			else
				RPT_Flag = 0;				//no stop condition for repeated start, reset flag

			//_bic_SR_register_on_exit(CPUOFF); //exit LPM0
		}
	}
	else
	{
		while(1);  							//trapp CPU //###for debugging
	}
}



